package main

import (
	"crypto/tls"
	"flag"
	"fmt"
	"log"
	"net/http"
	"path/filepath"
	"time"
)

// Constants for version and help text
const version = "1.0.0"
const helpText = `Usage: https_server [options] [path]
Options:
	-b, --bind      The address to bind the HTTPS server (default "localhost:4443").
  -v, --version   Display the version of the server.
  -h, --help      Display this help message.
Path:
  The path of the directory to serve. Defaults to the current directory.`

// Embed your certificate and key here
const (
	certPEM = `-----BEGIN CERTIFICATE-----
MIIDazCCAlOgAwIBAgIUQakPTNRUYCZPWH34izgrE1UV9sIwDQYJKoZIhvcNAQEL
BQAwRTELMAkGA1UEBhMCQVUxEzARBgNVBAgMClNvbWUtU3RhdGUxITAfBgNVBAoM
GEludGVybmV0IFdpZGdpdHMgUHR5IEx0ZDAeFw0yNDAxMjgwMjAyMjlaFw0yNTAx
MjcwMjAyMjlaMEUxCzAJBgNVBAYTAkFVMRMwEQYDVQQIDApTb21lLVN0YXRlMSEw
HwYDVQQKDBhJbnRlcm5ldCBXaWRnaXRzIFB0eSBMdGQwggEiMA0GCSqGSIb3DQEB
AQUAA4IBDwAwggEKAoIBAQCyiHdpxEAxxzlX36amxQYpQxLHsEeL5nV3GR8tqL1Q
KlfMZntzHnBVlbI2Lq20EjXerAZOfFwN6npgV/jrAjhwIUvzo4r99fAkSipI5fo4
o0cLGDaIhl8wNgmxxxCTWK2TF8tPxWuWlCC/MlUpKLcWUzsWCSFuykf31tGugOJ8
3TdxqDR5rsgl2rv43gppuN5WAQ3p1ekhyZCKh6QbK5wxbp4yzOalfl/27+D4/Tw6
s518/IGfp1YArwWYird3x4M8VbcSGk8g5gGEjC8lQqx7HaOfeywsmnIyOGg+WTgB
OUEj/ZoI/Y2FJm5C8WJmZuU0fRXeihF9ouiFiojeKsWhAgMBAAGjUzBRMB0GA1Ud
DgQWBBQpzM7JYyA0IYurS0M3MH9RSTumbjAfBgNVHSMEGDAWgBQpzM7JYyA0IYur
S0M3MH9RSTumbjAPBgNVHRMBAf8EBTADAQH/MA0GCSqGSIb3DQEBCwUAA4IBAQCN
M4wQ1fcBk6SWwO7F57o7lmIIIxRkp5WBg4f53pLxmkAkWGzItJwok294zNk4cFke
GnkqtB32ho1AEs7tONRgP6e/F3xbmYKSdAzM57IDVVwynQDQgAPl4ZN6ZG12JcXc
72qUnhc8BCQXvHwuir63MIIV1gTPkW2AHrWrCFTSlMvh+SW6d1owqGgsZ4+UL4WU
UrC2hyYlZBPuA7Hl0Ua+PJLY5Fk6b1eYpB3dLyqlnsdPu4J2sr960zC0BRDbIbwb
dkutiPdZYCg1Y+kOvGPjYaNkrHgF0sIBurRQ7BEOiuumtjfvDn7Z9KmaXus75zv9
UirvDyWPl6IFOLR/zIOw
-----END CERTIFICATE-----`

	keyPEM = `-----BEGIN PRIVATE KEY-----
MIIEuwIBADANBgkqhkiG9w0BAQEFAASCBKUwggShAgEAAoIBAQCyiHdpxEAxxzlX
36amxQYpQxLHsEeL5nV3GR8tqL1QKlfMZntzHnBVlbI2Lq20EjXerAZOfFwN6npg
V/jrAjhwIUvzo4r99fAkSipI5fo4o0cLGDaIhl8wNgmxxxCTWK2TF8tPxWuWlCC/
MlUpKLcWUzsWCSFuykf31tGugOJ83TdxqDR5rsgl2rv43gppuN5WAQ3p1ekhyZCK
h6QbK5wxbp4yzOalfl/27+D4/Tw6s518/IGfp1YArwWYird3x4M8VbcSGk8g5gGE
jC8lQqx7HaOfeywsmnIyOGg+WTgBOUEj/ZoI/Y2FJm5C8WJmZuU0fRXeihF9ouiF
iojeKsWhAgMBAAECgf8eVFp+0Qb5RoP504dNcAjA10kg2Y8mo+7fcax/NTyRmniG
EifX8weJ0Vg3nDGIsiL5mvX3UKQ1TLF4k++G1bTPktiDF74RYdt7OxRlIays2ejE
Gb3bKdFVpsTIsUsr5F1dYjm+Bzqf0raf6/lfZKZnHSQCsHuicb/DP8efWyNtq0qD
byVfc8oYTZzNuqUY0iOkBQhuol3TM+G0yggJpZ9ifzYshVbJnVN/Tz3ugsLatJ1w
QCG5pqf4LLLfRJqni5HVYUB4IrCGMsl0DBba3uqUCZ4GTa6zmn49nl7gJSTjPkNg
9OTyH1+aYYczr+c7GwI2PzIMH0eg8caDezSjGWkCgYEA6XPFtFXZbvKSzIuJOckj
zmRNUdXtfdRFJiSj837cMs89hBx1gEL1X6T6m1lT0ptM3/ZX1jWIndZJL+PStEN5
GXN65wXPpSclaQaEfOT+WohHmsLu+WJ+YQeyBofov/PBYw6A+muYkQOVZvGlnWvm
A85wpdMO6spqXeRcwNGEfAkCgYEAw8bKEsshFeiIpxYx2zdzsXqzONLo3lOeeuVA
apjiFHeeZPtVV8co2SyKFEOO678uGFAjGJF9iwapN6GOxyfhjfyLTBi6P8Ka0s10
D2x63rqQgt0goQQVJdYn/2Btv+k6h/UQJ/jClzkmTUvXLYxApR6WaBZKE/Xghwch
vLY9EtkCgYAmPVeCHZnbKZLQPH8C3yalVRqxL/iR5uZYxCGy7fHFxNqPvFWm59Dn
lM+UCMLJObUS7nge13AEYqhkVs4Zxv+cIqVcGECWDd574JxtFNlxHOeVux6H7RFE
dY08sqB2aMghoKuR2XQJNOwRC74UFit8LiGXmAXWgceAj7p9vxQTeQKBgQCe62w3
KzrVPOhIscSQzYeVhyOaueIcL0aTPis3HJlQwfUKxdZ2JY3sFLKVVm3awlsZk1uZ
4uhFBYgxR2zOD3qRtnIguGXfwgnJmstehdGLoWgTQortCZJdH2VicRVF1n4TxQNz
XwQem16TGkA2kgYbwyOWpJlHcKztDwX82PXkqQKBgDGA2+WHWDG9tMlitBohY9LN
URPkM1sFMR5iWih+Xqloc0OPao/SLDPYtW60UvimvZtYZDDSP7SIW6q5W9+W26/p
u1HrpmjerpibzLer9msspLIHsH55UqdzRRUjZOGvxPY03dPccmoNr4YeTVpHEOwF
Xdp3eR9NgJnXRlv+1XM6
-----END PRIVATE KEY-----`
)

// CustomResponseWriter is a wrapper around http.ResponseWriter
// that captures the status code of the HTTP response.
type CustomResponseWriter struct {
	http.ResponseWriter
	statusCode int
}

// NewCustomResponseWriter creates a new CustomResponseWriter.
func NewCustomResponseWriter(w http.ResponseWriter) *CustomResponseWriter {
	// Default the status code to 200, as WriteHeader might not be called explicitly
	return &CustomResponseWriter{w, http.StatusOK}
}

// WriteHeader captures the status code and delegates to the original writer.
func (w *CustomResponseWriter) WriteHeader(statusCode int) {
	w.statusCode = statusCode
	w.ResponseWriter.WriteHeader(statusCode)
}

// Custom logging handler
func loggingHandler(next http.Handler) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		// Log the incoming request
		start := time.Now()
		log.Printf("Started %s %s from %s", r.Method, r.URL.Path, r.RemoteAddr)

		customWriter := NewCustomResponseWriter(w)

		// Call the next handler
		next.ServeHTTP(customWriter, r)

		// Log the request and response details
		log.Printf("%s %s from %s - %d in %v", r.Method, r.URL.Path, r.RemoteAddr, customWriter.statusCode, time.Since(start))
	})
}

func main() {
	versionFlag := flag.Bool("version", false, "Display the version of the server")
	versionFlagShort := flag.Bool("v", false, "Display the version (short)")
	helpFlag := flag.Bool("help", false, "Display help message")
	helpFlagShort := flag.Bool("h", false, "Display help message (short)")
	bindAddr := flag.String("bind", "localhost:4443", "The address to bind the HTTPS server")
	bindAddrShort := flag.String("b", "localhost:4443", "The address to bind the HTTPS server (short)")
	flag.Parse()

	if *bindAddrShort != "localhost:4443" {
		bindAddr = bindAddrShort
	}

	// Handle version and help flags
	if *versionFlag || *versionFlagShort {
		fmt.Println("Version:", version)
		return
	}
	if *helpFlag || *helpFlagShort {
		fmt.Println(helpText)
		return
	}

	// Determine the directory to serve
	dir := "."
	if flag.NArg() > 0 {
		dir = flag.Arg(0)
	}
	dir, err := filepath.Abs(dir)
	if err != nil {
		log.Fatalf("Error determining absolute path: %v", err)
	}

	cert, err := tls.X509KeyPair([]byte(certPEM), []byte(keyPEM))
	if err != nil {
		log.Fatalf("Failed to parse certificate and key: %v", err)
	}

	// Create a file server handler
	fileServer := http.FileServer(http.Dir(dir))

	// Wrap the file server with the logging handler
	loggedFS := loggingHandler(fileServer)

	server := &http.Server{
		Addr: *bindAddr,
		TLSConfig: &tls.Config{
			Certificates: []tls.Certificate{cert},
			MinVersion:   tls.VersionTLS12,
		},
		Handler: loggedFS,
	}

	log.Printf("Serving %s at https://%s\n", dir, *bindAddr)
	log.Fatal(server.ListenAndServeTLS("", ""))
}
